@@ -114,11 +114,23 @@ impl CodeFormatter {
     }
 
     pub fn format_return<'ast>(&mut self, return_node: &Return<'ast>) -> Return<'ast> {
-        let formatted_returns =
+        let (mut formatted_returns, comments_buf) =
             self.format_punctuated(return_node.returns(), &CodeFormatter::format_expression);
+
         let wanted_token: TokenReference<'ast> = if formatted_returns.is_empty() {
             TokenReference::symbol("return").unwrap()
         } else {
+            // Append the comments buffer to the last return
+            if let Some(pair) = formatted_returns.pop() {
+                let pair = pair.map(|expr| {
+                    trivia_formatter::expression_add_trailing_trivia(
+                        expr,
+                        FormatTriviaType::Append(comments_buf),
+                    )
+                });
+                formatted_returns.push(pair);
+            }
+
             TokenReference::symbol("return ").unwrap()
         };
         let formatted_token = self.format_symbol(return_node.token(), &wanted_token);
