@@ -32,7 +32,7 @@ impl CodeFormatter {
     /// Format a GenericFor node
     pub fn format_generic_for<'ast>(&mut self, generic_for: &GenericFor<'ast>) -> GenericFor<'ast> {
         let for_token = crate::fmt_symbol!(self, generic_for.for_token(), "for ");
-        let formatted_names = self.format_punctuated(
+        let (formatted_names, mut names_comments_buf) = self.format_punctuated(
             generic_for.names(),
             &CodeFormatter::format_token_reference_mut,
         );
@@ -47,9 +47,19 @@ impl CodeFormatter {
             .collect();
 
         let in_token = crate::fmt_symbol!(self, generic_for.in_token(), " in ");
-        let formatted_expr_list =
+        let (formatted_expr_list, mut expr_comments_buf) =
             self.format_punctuated(generic_for.expr_list(), &CodeFormatter::format_expression);
+
+        // Create comments buffer and append to end of do token
+        names_comments_buf.append(&mut expr_comments_buf);
+
         let do_token = crate::fmt_symbol!(self, generic_for.do_token(), " do");
+        let do_token = Cow::Owned(trivia_formatter::token_reference_add_trivia(
+            do_token.to_owned().into_owned(),
+            FormatTriviaType::NoChange,
+            FormatTriviaType::Append(names_comments_buf),
+        ));
+
         let end_token = self.format_end_token(generic_for.end_token());
 
         let generic_for = generic_for
@@ -219,11 +229,7 @@ impl CodeFormatter {
 
         match stmt {
             Stmt::Assignment(assignment) => {
-                Stmt::Assignment(trivia_formatter::assignment_add_trivia(
-                    &assignment,
-                    leading_trivia,
-                    trailing_trivia,
-                ))
+                Stmt::Assignment(self.assignment_add_trivia(assignment, additional_indent_level))
             }
             Stmt::Do(do_block) => Stmt::Do(trivia_formatter::do_block_add_trivia(
                 do_block,
@@ -251,18 +257,12 @@ impl CodeFormatter {
                     trailing_trivia,
                 ))
             }
-            Stmt::If(if_block) => Stmt::If(trivia_formatter::if_block_add_trivia(
-                if_block,
-                leading_trivia,
-                trailing_trivia,
-            )),
-            Stmt::LocalAssignment(local_assignment) => {
-                Stmt::LocalAssignment(trivia_formatter::local_assignment_add_trivia(
-                    local_assignment,
-                    leading_trivia,
-                    trailing_trivia,
-                ))
+            Stmt::If(if_block) => {
+                Stmt::If(self.if_block_add_trivia(if_block, additional_indent_level))
             }
+            Stmt::LocalAssignment(local_assignment) => Stmt::LocalAssignment(
+                self.local_assignment_add_trivia(local_assignment, additional_indent_level),
+            ),
             Stmt::LocalFunction(local_function) => {
                 Stmt::LocalFunction(trivia_formatter::local_function_add_trivia(
                     local_function,
@@ -277,16 +277,12 @@ impl CodeFormatter {
                     trailing_trivia,
                 ))
             }
-            Stmt::Repeat(repeat_block) => Stmt::Repeat(trivia_formatter::repeat_block_add_trivia(
-                repeat_block,
-                leading_trivia,
-                trailing_trivia,
-            )),
-            Stmt::While(while_block) => Stmt::While(trivia_formatter::while_block_add_trivia(
-                while_block,
-                leading_trivia,
-                trailing_trivia,
-            )),
+            Stmt::Repeat(repeat_block) => {
+                Stmt::Repeat(self.repeat_block_add_trivia(repeat_block, additional_indent_level))
+            }
+            Stmt::While(while_block) => {
+                Stmt::While(self.while_block_add_trivia(while_block, additional_indent_level))
+            }
 
             #[cfg(feature = "luau")]
             Stmt::CompoundAssignment(compound_assignment) => {
